/*******************************************************}
{                                                       }
{       RichViewActions                                 }
{       Demo project for C++Builder 2009+               }
{       You can use it as a basis for your              }
{       applications.                                   }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "UnitUnicode1.h"
#include "RVALocRuler.hpp"
#include "ActionTestOptionsFrm.hpp"
//---------------------------------------------------------------------------


#include "GifImg.hpp"
#include "PngImage.hpp"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "PtblRV"
#pragma link "RichView"
#pragma link "RichViewActions"
#pragma link "Ruler"
#pragma link "RVEdit"
#pragma link "RVFontCombos"
#pragma link "RVRuler"
#pragma link "RVScroll"
#pragma link "RVStyle"
#pragma link "RVGifAnimate2007"
#pragma link "RVStyleFuncs"
#pragma resource "*.dfm"


#include "dmActionsImages1.hpp"
#include "dmActionsImages2.hpp"

TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

UnicodeString GetFullFileName(const UnicodeString FileName)
{
  UnicodeString s = ExtractFilePath(Application->ExeName)+FileName;
  if (FileExists(s))
	return s;
  s = ExtractFilePath(Application->ExeName)+"..\\"+FileName;
  if (FileExists(s))
	return s;
  s = ExtractFilePath(Application->ExeName)+"..\\..\\"+FileName;
  if (FileExists(s))
	return s;
  return "";
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
  cmbFont->Font->Name = RVAControlPanel1->DialogFontName;
  cmbFontSize->Font->Name = RVAControlPanel1->DialogFontName;
  cmbUnits->Font->Name = RVAControlPanel1->DialogFontName;
  cmbStyles->Font->Name = RVAControlPanel1->DialogFontName;

  // linking combo boxes and style inspector to RichViewEdit1
  UpdateLinkedControls();

  // Almost all these assignments could be done at design time in the Object Inspector
  // But in this demo we do not want to modify rvActionsResource
  // (and we recommend to use a copy of it in your applications)

// initializing TrvActionEditNote action
  rvActionEditNote1->Control = RichViewEdit1;
  rvActionEditNote1->OnFormCreate = rvActionEditNoteFormCreate;
  rvActionEditNote1->OnShowing = rvActionEditNoteShowing;
  rvActionEditNote1->OnHide = rvActionEditNoteHide;

  rvActionSave1->OnDocumentFileChange = rvActionSave1DocumentFileChange;

  // Code for making color-picking buttons stay pressed while a
  // color-picker window is visible.
  rvActionColor1->OnShowColorPicker = ColorPickerShow;
  rvActionColor1->OnHideColorPicker = ColorPickerHide;
  rvActionParaColor1->OnShowColorPicker = ColorPickerShow;
  rvActionParaColor1->OnHideColorPicker = ColorPickerHide;
  rvActionFontColor1->OnShowColorPicker = ColorPickerShow;
  rvActionFontColor1->OnHideColorPicker = ColorPickerHide;
  rvActionFontBackColor1->OnShowColorPicker = ColorPickerShow;
  rvActionFontBackColor1->OnHideColorPicker = ColorPickerHide;

  cmbUnits->ItemIndex = (int) RVAControlPanel1->UnitsDisplay;

  Localize();

  // Loading initial file via ActionOpen (allowing to update user interface)
  UnicodeString FileName = GetFullFileName("readme.rvf");
  if (FileName!="")
	rvActionOpen1->LoadFile(RichViewEdit1, FileName, ffiRVF);
  else
	rvActionNew1->ExecuteTarget(RichViewEdit1);
}
//---------------------------------------------------------------------------
/*                       Working with document                             */
//---------------------------------------------------------------------------
// When document is created, saved, loaded...
void __fastcall TfrmMain::rvActionSave1DocumentFileChange(TObject* Sender,
  TCustomRichViewEdit* Editor, const UnicodeString FileName,
  TrvFileSaveFilter FileFormat, bool IsNew)
{
  UnicodeString s = ExtractFileName(FileName);
  rvActionPrint1->Title = s;
  rvActionQuickPrint1->Title = s;
  if (IsNew)
    s += " (*)";
  Caption = s + " - RichViewActionsTest";
}
//---------------------------------------------------------------------------
// Prompt for saving...
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  CanClose = rvActionSave1->CanCloseDoc(RichViewEdit1);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mitExitClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
/*                  Working with color-picking buttons                     */
//---------------------------------------------------------------------------
// Code for making color-picking buttons pressed while
// a color-picker window is visible.
void __fastcall TfrmMain::ColorPickerShow(TObject* Sender)
{
  TComponent* ActionComponent = ((TAction*)Sender)->ActionComponent;
  if (ActionComponent->InheritsFrom(__classid(TToolButton)))
    ((TToolButton*)ActionComponent)->Down = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ColorPickerHide(TObject* Sender)
{
  TComponent* ActionComponent = ((TAction*)Sender)->ActionComponent;
  if (ActionComponent->InheritsFrom(__classid(TToolButton)))
    ((TToolButton*)ActionComponent)->Down = false;
}
//---------------------------------------------------------------------------
/*             Set of events for processing hypertext jumps                */
//---------------------------------------------------------------------------
// Hyperlink click
void __fastcall TfrmMain::RichViewEdit1Jump(TObject *Sender, int id)
{
  rvActionInsertHyperlink1->GoToLink(RichViewEdit1, id);
}
//---------------------------------------------------------------------------
// Inserting hyperlinks from files (RTF) and drag source
void __fastcall TfrmMain::RichViewEdit1ReadHyperlink(TCustomRichView *Sender,
  const UnicodeString Target, const UnicodeString Extras,
  TRVLoadFormat DocFormat, int &StyleNo, TRVTag &ItemTag,
  TRVRawByteString &ItemName)
{
  if (DocFormat==rvlfURL)
    StyleNo = rvActionInsertHyperlink1->GetHyperlinkStyleNo(RichViewEdit1);
  ItemTag = rvActionInsertHyperlink1->EncodeTarget(Target);
}
//---------------------------------------------------------------------------
// URL detection on typing
void __fastcall TfrmMain::RichViewEdit1KeyPress(TObject *Sender, wchar_t &Key)
{
  /*
  // uncomment if you use Addict3
  if (Key=='\'' || Key==13 || Key==9 || RichViewEdit1->Delimiters.Pos(Key))
	RVA_Addict3AutoCorrect(RichViewEdit1);
  */
  if (Key==13 || Key==9 || Key==' ' || Key==',' || Key==';')
  {
	rvActionInsertHyperlink1->DetectURL(RichViewEdit1);
	rvActionInsertHyperlink1->TerminateHyperlink(RichViewEdit1);
  }
}
//---------------------------------------------------------------------------
/*                     Insert table popup                                  */
//---------------------------------------------------------------------------
/* We use a trick: insert-table button has style tbsDropDown and assigned
  DropDownMenu (pmFakeDropDown). This menu is empty, but shows table size
  popup instead of itself */
void __fastcall TfrmMain::pmFakeDropDownPopup(TObject *Sender)
{
  rvActionInsertTable1->ShowTableSizeDialog(RichViewEdit1,
	ToolButton12);
}
//---------------------------------------------------------------------------
/*                             Ruler                                       */
//---------------------------------------------------------------------------
void __fastcall TfrmMain::RVAControlPanel1MarginsChanged(TrvAction *Sender,
	  TCustomRichViewEdit *Edit)
{
  RVRuler1->UpdateRulerMargins();
}
//---------------------------------------------------------------------------
/*                           Localization                                  */
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnLanguageClick(TObject *Sender)
{
  if (RVA_ChooseLanguage())
    Localize();
}
//---------------------------------------------------------------------------
void TfrmMain::Localize()
{
  // Localizing all actions on this form
  RVA_LocalizeForm(this);
  // Localizing ruler
  RVALocalizeRuler(RVRuler1);
  // Localizing units
  int Idx = cmbUnits->ItemIndex;
  RVA_TranslateUnits(cmbUnits->Items);
  cmbUnits->ItemIndex = Idx;
  for (int i=0; i<CoolBar1->Bands->Count; i++)
    if (CoolBar1->Bands->Items[i]->Control == tbUnits)
      CoolBar1->Bands->Items[i]->Text = RVA_GetS(rvam_lbl_Units);
  // Styles
  rvActionStyleInspector1->UpdateInfo(RichViewEdit1);
  cmbStyles->Localize();
  // Localizing menus
  mitFile->Caption   = RVA_GetS(rvam_menu_File);
  mitEdit->Caption   = RVA_GetS(rvam_menu_Edit);
  mitFont->Caption   = RVA_GetS(rvam_menu_Font);
  mitPara->Caption   = RVA_GetS(rvam_menu_Para);
  mitFormat->Caption = RVA_GetS(rvam_menu_Format);
  mitInsert->Caption = RVA_GetS(rvam_menu_Insert);
  mitTable->Caption  = RVA_GetS(rvam_menu_Table);
  mitTools->Caption  = RVA_GetS(rvam_menu_Tools);
  mitTextFlow->Caption  = RVA_GetS(rvam_menu_TextFlow);
  mitExit->Caption   = RVA_GetS(rvam_menu_Exit);

  mitFontSize->Caption = RVA_GetS(rvam_menu_FontSize);
  mitFontStyle->Caption = RVA_GetS(rvam_menu_FontStyle);
  mitTableSelect->Caption = RVA_GetS(rvam_menu_TableSelect);
  mitRotateCell->Caption = RVA_GetS(rvam_menu_TableCellRotation);
  mitTableCellBorders->Caption = RVA_GetS(rvam_menu_TableCellBorders);
  mitTableAlignCellContents->Caption = RVA_GetS(rvam_menu_TableCellAlign);
  // In your application, you can use either TrvActionFonts or TrvActionFontEx
  rvActionFonts1->Caption = rvActionFonts1->Caption+" (Standard)";
  rvActionFontEx1->Caption = rvActionFontEx1->Caption+" (Advanced)";

  /*
  // uncomment if you use Addict3. It's assumed that RVAddictSpell31
  // and RVThesaurus31 are on this form.
  RVAddictSpell31->UILanguage = GetAddictSpellLanguage(RVA_GetLanguageName());
  RVThesaurus31->UILanguage = GetAddictThesLanguage(RVA_GetLanguageName());
  */
}
//---------------------------------------------------------------------------
/*                          Progress messages                              */
//---------------------------------------------------------------------------
// Downloading
void __fastcall TfrmMain::RVAControlPanel1Download(TrvAction *Sender,
  const UnicodeString Source)
{
  if (Source.IsEmpty())
    Application->Hint = "";
  else
    Application->Hint = RVAFormat(RVA_GetS(rvam_msg_Downloading), ARRAYOFCONST((Source.w_str())));
}
//---------------------------------------------------------------------------
// Reading/writing
void __fastcall TfrmMain::RichViewEdit1Progress(TCustomRichView *Sender, TRVLongOperation Operation,
  TRVProgressStage Stage, BYTE PercentDone)
{
  switch (Stage)
  {
    case rvpstgStarting:
      ProgressBar1->Left = btnSkin->Left - ProgressBar1->Width - 8;
      ProgressBar1->Top  = btnSkin->Top + (btnSkin->Height-ProgressBar1->Height) / 2;
      ProgressBar1->Position = 0;
      ProgressBar1->Visible = true;
      Application->Hint = RVA_GetProgressMessage(Operation);
      break;
    case rvpstgRunning:
      ProgressBar1->Position = PercentDone;
      ProgressBar1->Update();
      break;
    case rvpstgEnding:
      ProgressBar1->Position = 100;
      ProgressBar1->Update();
      Application->Hint = "";
      ProgressBar1->Visible = false;
      break;
  }
}
//---------------------------------------------------------------------------
// Printing
void __fastcall TfrmMain::RVPrint1SendingToPrinter(TCustomRichView *Sender, int PageCompleted,
  TRVPrintingStep Step)
{
  Application->Hint = RVA_GetPrintingMessage(PageCompleted, Step);
}
//---------------------------------------------------------------------------
/*                              Notes                                      */
//---------------------------------------------------------------------------
void __fastcall TfrmMain::rvActionEditNoteFormCreate(TrvAction* Sender, TForm* Form)
{
  rvActionEditNote1->SubDocEditor->OnEnter = NoteEditorEnter;
  rvActionEditNote1->SubDocEditor->OnExit = NoteEditorExit;
  rvActionEditNote1->SubDocEditor->PopupMenu = RVAPopupMenu1;
  Form->Align = alBottom;
  Form->Height = 100;
  Form->Parent = this;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::rvActionEditNoteShowing(TrvAction* Sender, TForm* Form)
{
  RichViewEdit1->ForceFieldHighlight = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::rvActionEditNoteHide(TObject* Sender)
{
  RichViewEdit1->ForceFieldHighlight = false;
  if (RichViewEdit1->CanFocus())
    RichViewEdit1->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::NoteEditorEnter(TObject* Sender)
{
  RVAControlPanel1->DefaultControl = rvActionEditNote1->SubDocEditor;
  UpdateLinkedControls();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::NoteEditorExit(TObject* Sender)
{
  RVAControlPanel1->DefaultControl = RichViewEdit1;
  UpdateLinkedControls();
}
//---------------------------------------------------------------------------
void TfrmMain::UpdateLinkedControls()
{
  rvActionStyleInspector1->Control = RVAControlPanel1->DefaultControl;
  cmbFont->Editor = RVAControlPanel1->DefaultControl;
  cmbFontSize->Editor = RVAControlPanel1->DefaultControl;
  cmbStyles->Editor = RVAControlPanel1->DefaultControl;
}
//---------------------------------------------------------------------------
/*                    Units of measurement                                 */
//---------------------------------------------------------------------------
void __fastcall TfrmMain::cmbUnitsClick(TObject *Sender)
{
// --- 1. Units displayed to the user
  // units for displaying in dialogs
  RVAControlPanel1->UnitsDisplay = (TRVUnits)cmbUnits->ItemIndex;
  // units for displaying in the ruler
  RVRuler1->UnitsDisplay = (TRulerUnits)cmbUnits->ItemIndex;
  // units for page setup dialog
  if (RVAControlPanel1->UnitsDisplay==rvuPixels ||
      RVAControlPanel1->UnitsDisplay==rvuCentimeters ||
      RVAControlPanel1->UnitsDisplay==rvuMillimeters)
    rvActionPageSetup1->MarginsUnits = rvpmuMillimeters;
  else
	rvActionPageSetup1->MarginsUnits = rvpmuInches;
// --- 2. units for internal measurement
  // units of measurement for RichViewActions
  if (RVAControlPanel1->UnitsDisplay==rvuPixels)
  {
	RVA_ConvertToPixels(this);
	RVAControlPanel1->UnitsProgram = rvstuPixels;
  }
  else
  {
    RVA_ConvertToTwips(this);
    RVAControlPanel1->UnitsProgram = rvstuTwips;
  }
  // units of measurement for the current document
  if (RVStyle1->Units != RVAControlPanel1->UnitsProgram)
  {
    RichViewEdit1->ConvertDocToDifferentUnits(RVAControlPanel1->UnitsProgram);
    RVStyle1->ConvertToDifferentUnits(RVAControlPanel1->UnitsProgram);
    RichViewEdit1->Change();
  }
  RichViewEdit1->SetFocus();
}
//---------------------------------------------------------------------------
/*                    Tools                                                */
//---------------------------------------------------------------------------
void TfrmMain::ChangeImages(TCustomImageList* ilNormal, TCustomImageList* ilDisabled)
{
  for (int i = 0; i<ComponentCount; i++)
	if (Components[i]->InheritsFrom(__classid(TToolBar)))
	{
	  ((TToolBar*)(Components[i]))->Images = ilNormal;
	  ((TToolBar*)(Components[i]))->DisabledImages = ilDisabled;
	}
  MainMenu1->Images = ilNormal;
  RVAPopupMenu1->Images = ilNormal;
  rvActionStyleTemplates1->Images = ilNormal;
  rvActionStyleInspector1->Images = ilNormal;
  rvActionStyleInspector1->UpdateInfo();
  cmbStyles->Images = ilNormal;
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::mitImages1Click(TObject *Sender)
{
  ChangeImages(rvActionsImages1->ilNormal, rvActionsImages1->ilDisabled);
  mitImages1->Checked = true;
  mitImages2->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mitImages2Click(TObject *Sender)
{
  ChangeImages(rvActionsImages2->ilNormal, rvActionsImages2->ilDisabled);
  mitImages1->Checked = false;
  mitImages2->Checked = true;
}

//---------------------------------------------------------------------------

void __fastcall TfrmMain::mitOptionsClick(TObject *Sender)
{
  // This form is not a part of RichViewActions, it is only included in this demo.
  // It is not localized.
  TfrmOptions * frm = new TfrmOptions(Application);
  __try
  {
	frm->cmbTextEngine->ItemIndex = (int) (RVStyle1->TextEngine);
	frm->cbMetafilePrint->Checked = RVPrint1->MetafileCompatibility;
	if (frm->ShowModal()==mrOk)
	{
	  RVPrint1->MetafileCompatibility = frm->cbMetafilePrint->Checked;
	  if (frm->cmbTextEngine->ItemIndex != RVStyle1->TextEngine)
	  {
		RVStyle1->TextEngine = (TRVTextEngine) (frm->cmbTextEngine->ItemIndex);
		RichViewEdit1->Reformat();
	  }
	}
  }
  __finally
  {
	delete frm;
  }
}

//---------------------------------------------------------------------------
/*                    Live spelling with Addict 3                          */
//---------------------------------------------------------------------------
/*
// Add these events if you use Addict3
// (Assuming that you have RVAddictSpell31: TRVAddictSpell3 on the form)

// RichViewEdit1->OnSpellingCheck event
void __fastcall TfrmMain::RichViewEdit1SpellingCheck(TCustomRichView* Sender,
  const UnicodeString AWord, int StyleNo, bool& Misspelled)
{
  Misspelled = ! (RVAddictSpell31->WordAcceptable(AWord));
}

// RVAddictSpell31->OnParserIgnoreWord: if Ignore All or Add buttons were pressed
// in spellchecker dialog, removing underlines from the ignored word
void __fastcall TfrmMain::RVAddictSpell31ParserIgnoreWord(TObject* Sender,
  bool Before, int State)
{
  if (!Before && (State==IgnoreState_IgnoreAll || State==IgnoreState_Add))
	RichViewEdit1->LiveSpellingValidateWord(RVAddictSpell31->CurrentWord);
}

// Besides, if you want spelling check starting when loading document, call
// RichViewEdit1.StartLiveSpelling in rvActionFileOpen.OnOpenFile event
*/

//---------------------------------------------------------------------------
/*                       Visual Styles (C++Builder XE2+)                       */
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnSkinClick(TObject *Sender)
{
  RVA_ChooseStyle();
}

//---------------------------------------------------------------------------

